﻿Module MyGapSequences

    ' Ein paar Gap-Sequenzen zur Verfügung stellen
    Public GapSequences As GapSequence() = {New GapSequence("Shell (Original)", Function(i) MakeSequence(i, Function(x) 2 * x)), _
                                             New GapSequence("Sedgewick", AddressOf SedgewickSequence), _
                                             New GapSequence("Papernov-Stasevich", Function(i) MakeSequence(i, Function(x) 2 * x + 1)), _
                                             New GapSequence("Knuth", Function(i) MakeSequence(i, Function(x) 3 * x + 1)), _
                                             New GapSequence("11/5 x + 1", Function(i) MakeSequence(i, Function(x) x * 11 / 5 + 1)), _
                                             New GapSequence("InsertionSort", AddressOf InsertionSortSequence)}

    Public Delegate Function GapFunction(ByVal Size As Integer) As Integer()

    Public Function MakeSequence(ByVal Size As Integer, ByVal Operation As Func(Of Double, Double)) As Integer()
        Dim x = 1

        Dim Values = New List(Of Integer)()

        Do
            Values.Add(x)
            x = CInt(Operation(CDbl(x)))
        Loop Until x > Size

        Return DirectCast(Values, IEnumerable(Of Integer)).Reverse().ToArray()
    End Function

    Structure GapSequence
        Public Name As String
        Public Func As GapFunction

        Public Sub New(ByVal Name As String, ByVal Func As GapFunction)
            Me.Name = Name
            Me.Func = Func
        End Sub
    End Structure

    Public Function InsertionSortSequence(ByVal Size As Integer) As Integer()
        Return New Integer() {1}
    End Function

    Public Function SedgewickSequence(ByVal Size As Integer) As Integer()
        Dim p = 1, x = 1

        Dim Values = New List(Of Integer)()

        Do
            Values.Add(x)
            x = 4 ^ p + 3 * 2 ^ (p - 1) + 1
            p = p + 1
        Loop Until x > Size

        Return DirectCast(Values, IEnumerable(Of Integer)).Reverse().ToArray()
    End Function

End Module

